package moe.oko.Kiafumi;

import moe.oko.Kiafumi.model.KiafumiDB;
import org.simpleyaml.configuration.ConfigurationSection;
import org.simpleyaml.configuration.file.YamlConfiguration;

import java.util.List;

import static moe.oko.Kiafumi.Kiafumi.error;
import static moe.oko.Kiafumi.Kiafumi.info;

/**
 * KiafumiConfig class
 * Helps out with loading things from config and fetching them in a easy manner.
 */
public class KiafumiConfig {

    //Our nice YamlConfiguration class.
    public YamlConfiguration configuration;

    /*
    Discord Variable Section
     */
    private String token;
    private String logChannel;
    private String ownerId;
    private String mainGuild;
    private String clientId;
    private String prefix;
    private int defaultInvitePermissionLevel;

    /*
    Kia Variable Section
     */
    private boolean sharded;
    private String activityType;
    private String activityMsg;
    private String statusType;
    private List<String> pingResponses;

    /*
    SQL Variable Section
     */
    private String host;
    private int port;
    private String username;
    private String password;
    private String database;

    public KiafumiConfig(YamlConfiguration configuration) {
        //Load config on class creation...
        this.configuration = configuration;
    }

    public boolean load() {
        try {
            //Discord loaders
            info("Starting Discord Configuration load");
            ConfigurationSection discord = configuration.getConfigurationSection("discord");
            token = discord.getString("token");
            info("!!! DO NOT SHARE !!! Token - " + token);
            logChannel = discord.getString("logChannel");
            info("Logging Channel - " + logChannel);
            ownerId = discord.getString("ownerId");
            info("Owner ID - " + ownerId);
            mainGuild = discord.getString("mainGuild");
            info("Main Guild - " + mainGuild);
            clientId = discord.getString("clientId");
            defaultInvitePermissionLevel = discord.getInt("invitePermissionLevel");
            info("Invite link - " + assembleDefaultInvite());
            prefix = discord.getString("prefix");
            info("Prefix - " + prefix);
            //Shiki loaders
            ConfigurationSection main = configuration.getConfigurationSection("main");
            sharded = main.getBoolean("sharded");
            activityType = main.getString("activityType");
            activityMsg = main.getString("activityMsg");
            statusType = main.getString("statusType");
            pingResponses = main.getStringList("pingResponses");
            //SQL loaders
            ConfigurationSection sql = configuration.getConfigurationSection("sql");
            host = sql.getString("host");
            port = sql.getInt("port");
            username = sql.getString("username");
            password = sql.getString("password");
            database = sql.getString("database");
        } catch(Exception ex) {
            ex.printStackTrace();
            error("Failed to load configuration!");
            return false;
        }
        info("Configuration Loaded");
        return true;
    }

    public String assembleDefaultInvite() {
        return "https://discord.com/oauth2/authorize?client_id=" + clientId + "&scope=bot+applications.commands&permissions=" + defaultInvitePermissionLevel;
    }

    public String getPrefix() {
        return prefix;
    }

    public String getToken() { return token; }

    public String getActivityType() { return activityType; }

    public String getActivityMsg() { return activityMsg; }

    public String getStatusType() { return statusType; }

    public String getLogChannel() { return logChannel; }

    public List<String> getPingResponses() { return pingResponses; }

    public KiafumiDB createDb() { return new KiafumiDB(username, password, host, port, database); }
}
